package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import akka.actor.ActorRef;
import akka.actor.ActorSystem;
import akka.actor.Props;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.actor.MesHandlerEquipLogActor;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base.MesEquipmentExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.equiplog.MesEquipmentLogExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesSnProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.ActorMessage;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.config.SpringExt;
import cn.estsh.i3plus.mes.pcn.config.SpringExtProvider;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentLogDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentLogRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.milo.opcua.stack.core.channel.messages.HelloMessage;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @Description : 保存工艺参数
 * @Author : zxw
 **/
@Slf4j
@Service
public class MesVariableWhenFinishedReadStepService extends BaseStepService {
    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private ActorSystem actorSystem;

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;



    @Autowired
    private IMesProductionDispatchContextStepService mesProductionDispatchContextStepService;

    public static final String ORDER_NO_JIS_SORT = "ORDER_NO_JIS_SORT";

    private static final Map<String, ActorRef> refMap = new ConcurrentHashMap<>(200);

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        // 获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getCurCellEquipment(reqBean);

        // 获取条码信息
        List<MesProduceSn> mesProduceSns = mesProductionDispatchContextStepService.getOutProduceSnDataContext(reqBean);
        // 通过上下文获取工位设备信息
        MesCellEquipContext curCellEquip = productionProcessContext.getCurCellEquip();

        String mesProduceSnsStr = !CollectionUtils.isEmpty(mesProduceSns) ? JSONObject.toJSONString(mesProduceSns) : null;
        ActorMessage actorMessage = ActorMessage.builder()
                .equipmentCode(curCellEquip.getEquipmentCode())
                .equipId(curCellEquip.getEquipId())
                .organizeCode(reqBean.getOrganizeCode())
                .organizeName(reqBean.getOrganizeName())
                .workCellCode(reqBean.getWorkCellCode())
                .workCenterCode(reqBean.getWorkCenterCode())
                .areaCode(reqBean.getAreaCode())
                .craftCode(productionProcessContext.getCraftCode())
                .processCode(reqBean.getProcessCode())
                .produceSnJson(mesProduceSnsStr)
                .userName(reqBean.getUserInfo())
                .build();
        // 数据采集信息需要存放到 redis中
        String key = this.getFsmBusikey(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PROCESS_FINISH.name());
        ActorRef ref = refMap.computeIfAbsent(key,
                k -> actorSystem.actorOf(SpringExtProvider.getInstance().get(actorSystem).create("mesHandlerEquipLogActor"), key));
        ref.tell(actorMessage, ActorRef.noSender());

        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "保存工艺参数成功");
    }
}
